import { createRouter, createWebHistory } from 'vue-router';
import { useUserStore } from './store';

const router = createRouter({
    history: createWebHistory(),
    routes: [
        {
            path: '/',
            name: 'home',
            component: () => import('./components/pages/Home.vue'),
        },
        {
            path: '/login',
            name: 'login',
            component: () => import('./components/pages/Login.vue'),
            meta: { notRequiresAuth: true },
        },
        {
            path: '/signup',
            name: 'signup',
            component: () => import('./components/pages/SignUp.vue'),
            meta: { notRequiresAuth: true },
        },
        {
            path: '/groups',
            name: 'groups',
            component: () => import('./components/pages/Groups.vue'),
        },
        {
            path: '/groups/report/period',
            name: 'periodReport',
            component: () => import('./components/pages/PeriodReport.vue'),
        },
        {
            path: '/plans/:groupId',
            name: 'plans',
            component: () => import('./components/pages/Plans.vue'),
        },
        {
            path: '/plans/report/offset/:planId',
            name: 'offsetReport',
            component: () => import('./components/pages/OffsetReport.vue'),
        },
    ],
});

router.beforeEach((to, _, next) => {
    const store = useUserStore();
    if (!to.meta.notRequiresAuth && !store.user.id) {
        next({ name: 'login' });
    } else {
        next();
    }
});

export default router;